#!/usr/local/perl/bin/perl -w
#
# file: isip_play.perl
#
# This is a more robust version of the original play.sh by Dr. Picone
# which uses sox to convert an raw audio file into a sun audio file
# and play it over ncd audio
#


$sample_frequency = 16000;
$volume = 1;
$device = "";
$num_chan = 1;
$found=0;
$quiet=0;
$type="";
$prog_name = "isip_play";
$swap_byte = 0;
$fswap_byte = 0;
$swap_word = 0;
$swap_stuff = "";
$file_name = "";
foreach $word (@ARGV) {
    if ($found == 1){
	$sample_frequency = $word;
	$found=0;
    } 
    elsif  ($found == 2) {
	$volume = $word;
	$found=0;
    }
    elsif ($found == 3) {
	$device = $word;
	$found=0;
    }
    elsif ($found == 4) {
	$num_chan = $word;
	$found=0;
    }
    elsif ($found == 5) {
	$type = $word;
	$found=0;
    } else {
	
	if ($word eq "-quiet") {
	    $quiet = 1;
	}
	elsif ($word eq "-device") {
	    $found = 3;
	}  
	elsif ($word eq "-sf") {
	    $found = 1;
	} 
	elsif ($word eq "-v") {
	    $found = 2;
	} 
	elsif ($word eq "-debug") {
	    $verbose = 1;
	}
	elsif ($word eq "-num_chan") {
	    $found = 4;
	}
	elsif ($word eq "-t") {
	    $found = 5;
	}
	elsif ($word eq "-swap_word") {
	    $swap_word = !($swap_word);
	}
	elsif ($word eq "-swap_byte") {
	    $swap_byte = !($swap_byte);
	}
	elsif ($word eq "-fswap_byte") {
	    $fswap_byte = !($fswap_byte);
	}
	elsif ($word eq "-help") {
	    $help = 1;
	    print "name: $prog_name\n";
	    print "synopsis: $prog_name file [options]\n";
	    print "descr: plays audio files over the ncd audio system\n";
	    print "example: $prog_name file.raw -sf 16000\n";
	    print "\noptions:\n";

	    print "-sf: sample frequency (default 16000)\n";
	    print "-num_chan: number of audio channels (1)\n";
	    print "-v: volume amp. factor (default 1)\n";
	    print "-device: audio_system to send audio to (default NULL)\n";
	    print "-dev_type: (see below for default) either DAT, NCD, or PC\n";
	    print "-t: type of file (default extrapolated from file extension)\n";
	    print "-byte_swap: default none, converts 0123->2132\n";
	    print "-word_swap: default none, converts 0123->2301\n";
	    print "-quiet: suppress all standard output\n";
	    print "-debug: print full debug info, overrides -quiet\n";
	    print "-help: display this help message\n\n";

	    print "currently, we handle sphere (pcm or ulaw), au, ul, and raw data\n\n";
	    print "$prog_name tries to determine dev_type from the device itself.\n";
	    print "If it does not work properly, use the -dev_type to force correct operation.\n\n";
	    
	    
	    print "\nman page: none\n";
	}
	else {
	    $file_name = $word;
	}
    }
}


if(!($help)){

    if ($quiet && $verbose) {
	print "WARNING, both verbose and quiet flags given, being verbose\n";
	$quiet = 0;
    }


    if (!($file_name)) {
	print "ERROR: Enter a filename\n        -help for help message\n";
    } else {

	# find file format
	#
	$pos = rindex($file_name,".") +1;
	if ($type eq "") {
	    $ext = substr($file_name,$pos);
	}
	else {
	    $ext = $type;
	}

		
	if($file_name eq "-"){
	    $pos=1;
	    if ($type eq "") {
		print "ERROR: enter file type for stdin\n";
		$pos = -1;
	    }
	    open(PIN,"cat|");
	} else {
	    open(PIN,$file_name);
	}

	
	if ($pos > 0) {
	    if ($ext eq "raw"){
		$in_format = "sw";
	    } else {
		if ($ext eq "sphere"){
		    $header="";
		    if (sysread(PIN,$header,1024) != 1024) {
			print "ERROR: invalid sphere file\n";
			exit;
		    }
		    $_ = $header;
		    if (!(/NIST_1A(.*)/)) {
			print "Error: invalid sphere header";
			exit;
		    }
		    if (/.*sample_rate -i (.*)/) {
			$sample_frequency = $1;
		    } else {
			print "SPHERE: no sample_rate\n";
		    }
		    if (/.*sample_byte_format -s. (.*)/) {
			if ($1 eq "10") {
#			    print "standard MSB-LSB word config\n";
			} elsif ($1 eq "01") {
			    $fswap_byte = !($fswap_byte);
			} elsif ($1 eq "1") {
			    
			} else {
#			    print "SPHERE: no sample_byte_format\n";
			}
		    } else {
			print "SPHERE: no sample_byte_format\n";
			exit;
		    }

		    if (/.*channel_count -i (.*)/) {
			$num_chan = $1;
		    }
		    if (/.*sample_coding -s. (.*)/) {
			if ($1 eq "pcm") {
			    $in_format = "sw";
			} elsif ($1 eq "ulaw") {
			    $in_format = "ul";
			} else {
			    print "SPHERE: no sample_coding\n";
			}
#			print "\n\nsample_coding = $1 -> $in_format\n\n";

		    } else {
			print "SPHERE: no sample_coding\n";
		    }
		    
		    
		} else {
		    $in_format = $ext;
		}
	    }
	}

	$_ = $ENV{'ISIP_BINARY'};
	if (/i386.*/) {
	    $_ = $in_format;
	    if (!(/au/)) {
		$swap_byte = !($swap_byte);
	    }
	}
	
	if ($swap_byte) {
	    $swap_stuff = "-x";
	}

	$_ = $ENV{'DISPLAY'};
	if ($_ eq ":0.0") {
	    $_ = "$ENV{'ISIP_HOST'}$_";
	}
	if (/(ncd..c|isip04).*/) {
	    if ($device eq "") {
		$device = $_;
	    }
	}

	if ($device eq "") {
	    $device = $ENV{'ISIP_AUDIO_DEVICE'};
	}
	$_ = $device;

	$volume_used = 0;

	if (/isip02:[01]/) {
	    # use the naplay command instead of auplay
	    $play = "| naplay >/dev/null -s $sample_frequency -u $device -o mono -f raw 2>/dev/null";
	    $out_format = ".sw";
	} else {
	    # use auplay
	    $out_format = ".au";

	    $volume_per = 100 * $volume;

	    if ($device =~ /isip04/) {
		$volume_per = int($volume_per/3);
		if ($volume_per > 99) {
		    $volume_per = 99;
		}
		$play = "|audioplay -v $volume_per";
		$volume_used = 1;
	    } else {
		$play = "| auplay -audio $device -volume $volume_per";
		$volume_used = 1;
	    }
	}
	
	if ($pos > 0){
	    if (!($volume_used)) {
		$sox_part = "| sox $swap_stuff -r $sample_frequency  -c $num_chan -v $volume -t .$in_format - -t $out_format -";
	    } else {
		$sox_part = "| sox $swap_stuff -r $sample_frequency  -c $num_chan -t .$in_format - -t $out_format -";
	    }
	    if (!($quiet)){
		print $sox_part;
		#print $command_line,"\n";
	    }

	    if ($verbose){
		print "SAMPLE FREQUENCY = ",$sample_frequency,"\n";
		print "VOLUME = ",$volume,"\n";
		print "DEVICE = ",$device,"\n";
		print "NUM_CHAN = ",$num_chan,"\n";
		print "FILE = ", $file_name,"\n";
		print "SWAP_BYTE =",$swap_byte,"\n";
		print "SWAP_WORD =",$swap_word,"\n";
		print "FORCE_SWAP_BYTE =",$fswap_byte,"\n";
		print "FILE_TYPE =",$type,"\n";

		print "$sox_part --> $play\n";
	    }
	    

	    open(POUT,"$sox_part $play");
	    $len=0;
	    $buf = "";

	    # branch here on output byte ordering
	    #
	    # 8 bytes 01234567 ->
	    #
	    if (!$swap_word) {

		if (!$fswap_byte) {
		    # 8 bytes 01234567 -> 01234567 (no change)
		    #
		    while(($len = sysread(PIN,$buf,8192))>0){
			syswrite(POUT,$buf,$len);
		    } 
		} else {
		    # 8 bytes 01234567 -> 10325476 (swap bytes)
		    #
		    while(($len = sysread(PIN,$buf,8192))>0){
			$len2 = $len / 2;
			syswrite(POUT,
				 pack("v$len",
				      unpack("n$len",$buf))
				 ,$len);
		    } 
		}
	    } else {
		if (!$fswap_byte) {
		    # 8 bytes 01234567 -> 23016745 (swap words)
		    #
		    while(($len = sysread(PIN,$buf,8192))>0){
			$len2 = $len / 2;
			syswrite(POUT,
				 pack("v$len",
				      unpack("n$len",
					     pack("V$len2",
						  unpack("N$len2",$buf))))
				 ,$len);
		    } 
		
		} else {
		    # 8 bytes 01234567 -> 23016745 (swap words & bytes)
		    #
		    while(($len = sysread(PIN,$buf,8192))>0){
			$len2 = $len / 2;
			syswrite(POUT,
				 pack("V$len2",
				      unpack("N$len2",$buf))
				 ,$len);
		    } 
		}
	    }
	} else {
	    print "ERROR: invalid file format\n";
	}
    }
}
