// file: bm_file_5.cc
//

// system include files
//
#include <string.h>

// isip include files
//
#include "boltz_machine.h"
#include "boltz_machine_constants.h"

// method: expand_filename_cc
//
// arguments:
//  char_1*& fname: (input/output) name needing expansion
//
// return: a logical_1 value indicating status
//
// this method replaces all environment variables and other such
// things in a string with their equivalent expansions. in other
// words, variables or aliases anywhere in the string will be
// replaced. this method uses popen/pclose rather than complicated
// function calls and string replacements.
//
logical_1 Boltz_machine::expand_filename_cc(char_1*& fname_a) {

  // dummy variables
  //
  logical_1 flag_match = ISIP_FALSE;
  FILE* fp = (FILE*)NULL;
  
  // create space to store the expanded string
  //
  char_1* oname = new char_1[ISIP_MAX_FNAME_SIZE];

  // remove new-line characters
  //
  int_4 len = strlen((char*)fname_a);
  if (fname_a[len - 1] == ISIP_NEWLINE) {
    fname_a[len - 1] = ISIP_NULL;
    len--;
  }
  
  // scan the incoming string for special characters
  //
  for (int_4 i = 0; i < len; i++) {
    if (strchr((char*)BM_ENV_DELIMITERS, fname_a[i]) != (char*)NULL) {

      // set flag and break
      //
      flag_match = ISIP_TRUE;
      break;
    }
  }
  
  // if necessary, expand the string
  //
  while (flag_match == ISIP_TRUE) {

    // build a Unix command
    //
    sprintf((char*)oname, "echo %s\n", (char*)fname_a);
    
    // execute the pipe
    //
    fp = popen((char*)oname, "r");
    fgets((char*)oname, ISIP_MAX_FNAME_SIZE, fp);
    pclose(fp);

    // strip the trailing linefeed
    //
    len = strlen((char*)oname) - (int_4)1;
    oname[len] = ISIP_NULL;

    // copy this over to the input filename
    //
    if (fname_a != (char_1*)NULL) {
      delete [] fname_a;
    }    
    fname_a = new char_1[strlen((char*)oname) + 1];
    strcpy((char*)fname_a, (char*)oname);
    
    // make sure everything got expanded
    //
    flag_match = ISIP_FALSE;
    for (int_4 i = 0; i < len; i++) {
      if (strchr((char*)BM_ENV_DELIMITERS, fname_a[i]) != (char*)NULL) {
	flag_match = ISIP_TRUE;
	break;
      }
    }
  }

  // free memory
  //
  if (oname != (char_1*)NULL) {
    delete [] oname;
    oname = (char_1*)NULL;
  }
  
  // exit gracefully
  //
  return ISIP_TRUE;
}
