#! /usr/bin/sh
#
# file: install.sh
#
# this is the installation script for the ISIP pronunciations software
#

# check directory exists
#
if [ -z $ISIP_PRON ] 
    then
        echo "must set \$ISIP_PRON before proceeding."
        exit 3

else
    rm -f $ISIP_PRON/bin/$ISIP_BINARY/*
    rm -f $ISIP_PRON/lib/$ISIP_BINARY/*
fi

# prompt user
#
echo "Installing classes..."
echo ""

# set the header files
#
cd $ISIP_PRON/include
rm -f array.h
ln -s ../class/array/array.h array.h
rm -f boltz_machine.h
ln -s ../class/boltz_machine/boltz_machine.h boltz_machine.h
rm -f boltz_machine_constants.h
ln -s ../class/boltz_machine/boltz_machine_constants.h boltz_machine_constants.h
rm -f boltz_neuron.h
ln -s ../class/boltz_neuron/boltz_neuron.h boltz_neuron.h
rm -f boltz_random.h
ln -s ../class/boltz_random/boltz_random.h boltz_random.h
rm -f input_buffer.h
ln -s ../class/input_buffer/input_buffer.h input_buffer.h
rm -f output_pron.h
ln -s ../class/output_pron/output_pron.h output_pron.h
rm -f tree.h
ln -s ../class/tree/tree.h tree.h
rm -f tree_constants.h
ln -s ../class/tree/tree_constants.h tree_constants.h

# install each class in the respective library
#
cd $ISIP_PRON/class/array
make clean; make install -j5; make clean

cd $ISIP_PRON/class/boltz_machine
make clean; make install -j5; make clean

cd $ISIP_PRON/class/boltz_neuron
make clean; make install -j5; make clean

cd $ISIP_PRON/class/boltz_random
make clean; make install -j5; make clean

cd $ISIP_PRON/class/input_buffer
make clean; make install -j5; make clean

cd $ISIP_PRON/class/output_pron
make clean; make install -j5; make clean

cd $ISIP_PRON/class/tree
make clean; make install -j5; make clean

# prompt user
#
echo "Installing utilities..."
echo ""

# install various utilities
#
echo "Installing network building software..."
cd $ISIP_PRON/util/build_net
make clean; make install -j5; make clean

echo "Installing neural network software..."
cd $ISIP_PRON/util/generate_pron
make clean; make install -j5; make clean

echo "Installing decision tree software..."
cd $ISIP_PRON/util/generate_tree
make clean; make install -j5; make clean

echo "Installing network rebuild software..."
cd $ISIP_PRON/util/rebuild
make clean; make install -j5; make clean

#---------------------------------------------------------------------
# The text-to-speech software requires you to install rsynth (a public
# domain TTS system) as well as  the ISIP buffers around it. This also
# requires a  GNU package  called gdbm, which  is not included  in the
# current distribution.  If you wish to  not use the  TTS component of
# the demo, please comment out the lines below as indicated.
#---------------------------------------------------------------------

#--- begin comment section ---

# install text-to-speech software
#
echo "Installing text_to_speech using rsynth..."
cd $ISIP_PRON/util/rsynth
make clean; make -j5
cp text_to_speech $ISIP_PRON/bin/$ISIP_BINARY/text_to_speech
make clean

#--- end comment section ---

# install GUI software
#
echo "Installing GUI..."
echo "$ISIP_PRON/gui/tcl/main.tcl &" > $ISIP_PRON/bin/scripts/isip_pron_demo
chmod ugo+x $ISIP_PRON/gui/tcl/main.tcl
chmod ugo+x $ISIP_PRON/bin/scripts/isip_pron_demo

# prompt user
#
echo "The installation is complete now. You are welcome to run the demo."
echo "Please begin by typing the following command:"
echo ""
echo "isip_pron_demo"

#
# end of file
