// file: build_net.cc
//
// main function to generate a network of phonemes from the
// n-best pronunciations
//

// isip include files
//
#include "build_net.h"

// main function for building pronunciation network file
//
int main(int_4 argc, char_1** argv) {
  
  // verify arguments
  //
  if (argc != 4) {
    fprintf(stderr, "Usage: build_net <infile> <outfile> <n_best>\n");
    exit(ISIP_ERROR);
  }
  
  // open the input file and create a network
  //
  char_1* infile = argv[1];
  char_1* outfile = argv[2];
  FILE* inFile = fopen((char*)infile, "r");
  if (inFile == NULL) {
    fprintf(stderr, "Error in opening file %s.\n", infile);
    exit(ISIP_ERROR);
  }
  
  // read the data from the file
  //
  char_1* data_str  = new char_1[MAX_WORD_SIZE];
  char_1* phone_str = new char_1[MAX_PHONE_SIZE];
  
  // read the header
  //
  fscanf(inFile, "%[* ]", data_str);
  fscanf(inFile, "%[^\n]", data_str);
  int_4 word_length = strlen((char*)data_str);

  // create the arrays
  //
  int_4 n_best = (int_4)atoi((char*)argv[3]);
  Array** phone_matrix = new Array*[n_best];
  for (int_4 j = 0; j < n_best; j++) {
    phone_matrix[j] = new Array(j, word_length);
  }
  
  // read the phonemes one by one and store them in arrays
  //
  int_4 i;
  int_4 num_rows = 0;
  int_4 blank_rows = 0;
  int_4 count_rows = 0;
  char_1* temp_str = new char_1[MAX_PHONE_SIZE];
  
  while (fscanf(inFile, "%[\n ]", data_str) != EOF) {
    
    // read the phonemes
    //
    for (i = 0; i < word_length; i++) {
      
      if (fscanf(inFile, "%s", phone_str) == EOF) {
	break;
      }
      
      // check and insert phoneme in the appropriate array
      //
      count_rows = 0;
      blank_rows = 0;
      for (int_4 j = 0; j < num_rows + 1; j++) {
	if (phone_matrix[j]->check_phone_cc(i, phone_str)
	    == ISIP_FALSE) {
	  if (phone_matrix[j]->check_phone_cc(i, (char_1*)"#")
	      == ISIP_FALSE) {
	    count_rows ++;
	  }
	  else {
	    blank_rows ++;
	  }
	}
      }
      
      // now find a suitable place to insert this
      //
      if (count_rows + blank_rows == num_rows + 1) {
	int_4 j = 0;
	for (j = 0; j < num_rows + 1; j++) {
	  if (phone_matrix[j]->check_phone_cc(i, (char_1*)"#")
	      == ISIP_TRUE) {
	    phone_matrix[j]->set_phone_cc(i, phone_str);
	    break;
	  }

	  else if (j == num_rows) {
	    phone_matrix[++ num_rows]->set_phone_cc(i, phone_str);
	    break;
	  }
	}
      }
    }

    // read junk after the phones
    //
    if (fscanf(inFile, "%[^\n]", data_str) == EOF)
      break;
  }
  
  // reset the blanks
  //
  for (int_4 nn = 0; nn < word_length; nn++)  {
    for (int_4 ii = 1; ii < num_rows + 1; ii++) {
      if (phone_matrix[ii]->check_phone_cc(nn, (char_1*)"_")
	  == ISIP_TRUE) {
	phone_matrix[0]->get_phone_cc(nn, temp_str);
	phone_matrix[ii]->set_phone_cc(nn, temp_str);
	phone_matrix[0]->set_phone_cc(nn, (char_1*)"_");
      }
    }
  }
  
  // re-align the blanks
  //
  for (int_4 nn = 0; nn < word_length; nn++) {
    for (int_4 ii = 1; ii < num_rows + 1; ii++) {
      if (phone_matrix[0]->check_phone_cc(nn, (char_1*)"_")
	  == ISIP_TRUE) {
	if (phone_matrix[ii]->check_phone_cc(nn, (char_1*)"#")
	    == ISIP_TRUE) {
	  phone_matrix[ii]->set_phone_cc(nn, (char_1*)"_");
	}
      }
    }
  }

  // open file for output
  //
  FILE *outFile = fopen((char*)outfile, "w");
  if (outFile == NULL) {
    fprintf(stderr, "Error in opening file %s.\n", outfile);
    exit(ISIP_ERROR);
  }
  
  // write the arrays to output
  //
  for (int_4 ii = 0; ii < num_rows + 1; ii++) {
    phone_matrix[ii]->print_array_cc(outFile);
  }

  // close files
  //
  fclose(inFile);
  fclose(outFile);

  // free string memory
  //
  delete [] phone_str;
  delete [] data_str;
  delete [] temp_str;

  // free object memory
  //
  for (int_4 i = 0; i < n_best; i++) {
    delete phone_matrix[i];
    phone_matrix[i] = (Array*)NULL;
  }
  delete [] phone_matrix;
  phone_matrix = (Array**)NULL;

  // exit gracefully
  //
  exit(ISIP_NO_ERROR);
}
