// file: generate_pron.cc
// 
// main function for training and executing a boltzmann machine neural
// network
//

// isip include files
//
#include "generate_pron.h"

// main function for boltzmann machine
//
int main (int_4 argc, char_1** argv) {

  // various string variables for options
  //
  char_1* operation = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* prmsfile = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* datafile = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* outfile = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* wtfile = new char_1[ISIP_MAX_STRING_LENGTH];
  
  // dummy variables
  //
  int_4 nbest = 0;
  
  // check for commandline inputs being correct
  //
  for (int_4 ind = 0; ind < argc; ind ++) {
    
    // check for the help message
    //
    if ((argc == 1) || (strcmp((char*)ISIP_HELP_OPTION,
			       (char*)argv[ind]) == 0)) {
      
      // print the help message
      //
      char_1 cmd_str[ISIP_MAX_STRING_LENGTH];
      sprintf((char*)cmd_str, (char*)ISIP_HELP_FORMAT, (char*)GP_HELP_FILE);
      system((char*)cmd_str);
      
      // exit gracefully
      //
      exit(ISIP_ERROR);
    }
    
    // determine function --- training or testing
    //
    else if (strcmp(GP_FUNCTION_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)operation, (char*)argv[ind+1]);
    }
    
    // read the name of the parameter file
    //
    else if (strcmp(GP_PARAMS_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)prmsfile, (char*)argv[ind+1]);
    }
    
    // read the number of options (n-best) wanted
    //
    if (strcmp(GP_NBEST_OPT, (char*)argv[ind]) == 0) {
      nbest = (int_4)atoi((char*)argv[ind+1]);
    }
    
    // read the name of the training / testing datafile
    //
    if (strcmp(GP_INFILE_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)datafile, (char*)argv[ind+1]);
    }
    
    // read the name of the output file
    // this will be the log file for training
    // or the pronunciations output file for testing
    //
    if (strcmp(GP_OUTFILE_OPT, (char*)argv[ind]) == 0) {
      strcpy((char*)outfile, (char*)argv[ind+1]);
    }
  }
  
  // create a Boltzmann machine network and train or run it as specified
  //
  Boltz_machine boltz_network;
  
  // load appropriate parameters
  //
  boltz_network.set_params_cc(wtfile, operation, prmsfile, nbest);
  
  // train the machine
  //
  if (strcmp((char*)operation, "train") == 0) {
    boltz_network.train_cc (outfile, wtfile, datafile);
  }
  
  // run the machine
  //
  else if ((strcmp((char*)operation, "test") == 0) ||
	   (strcmp((char*)operation, "mlp") == 0)) {
    boltz_network.test_cc (outfile, wtfile, datafile);
  }

  // free memory
  //
  delete [] operation;
  delete [] prmsfile;
  delete [] datafile;
  delete [] outfile;
  delete [] wtfile;

  // exit gracefully
  //
  exit(ISIP_NO_ERROR);
}

//
// end of file
