// file: r_qsort_0.cc
//

// isip include files
//
#include "rebuild.h"
#include "rebuild_constants.h"

// method: compare_float_cc
//
// arguments:
//   float_4* num1_a: (input/output) 
//   float_4* num2_a: (input/output)
//
// return:
//   an int_4 to indicate status
//
// This method compares two numbers and returns status of comparison.
//
//
static int compare_float_cc(const void* num1_a, const void* num2_a) {

  int val = 0;
  if (*(float_4*)num1_a > *(float_4*)num2_a) {
    val = -1;
  }
  else if (*(float_4*)num1_a < *(float_4*)num2_a) {
    val = 1;
  }
  return val;
}

// method: qsort_cc
//
// arguments:
//  int_4* index: (output) buffer containing index array
//  float_4* values: (input) buffer containing the data to be sorted
//  int_4 num_items: (input) number of items in data buffer
//
// return:
//  logical_1 to indicate status
//
// This methods sorts the given buffer by returning an index array
// that will be used to index into the original array to get the
// sorted data.
//
logical_1 qsort_cc(int_4* index_a, float_4* values_a, int_4 num_items_a) {
  
  // copy the data into a temporary space
  //
  float_4 temp[num_items_a];
  memcpy(temp, values_a, num_items_a * sizeof(float_4));
  
  // sort the temp array
  //
  qsort(temp, num_items_a, sizeof(float_4), compare_float_cc);

  // compute an index array using a replacement algorithm
  //
  for (int_4 i = 0; i < num_items_a; i++) {
    for (int_4 j = 0; j < num_items_a; j++) {
      if (temp[i] == values_a[j]) {
	index_a[i] = j;
	temp[i] = ISIP_MINIMUM_FLOAT_4;
	break;
      }
    }
  }

  // exit gracefully
  //
  return ISIP_TRUE;
}
