// file: rebuild.cc
//

// system include files
//
#include <string.h>
#include <ctype.h>

// isip include files
//
#include "rebuild.h"
#include "rebuild_constants.h"

// main program
//
int main (int_4 argc, char_1* argv[]) {

  int_4 nbest = (int_4)0;
  int_4 max = (int_4)0;
  int_4 context = (int_4)0;
  char_1* fname_in = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* fname_out = new char_1[ISIP_MAX_STRING_LENGTH];
  
  // parse command line arguments
  //
  if (get_parameters_cc(nbest, max, context, fname_in, fname_out,
			argc, argv) != ISIP_TRUE) {
    fprintf(stdout, "<%s: error in command line arguments>\n", argv[0]);
    exit(ISIP_ERROR);
  }
  
  FILE* fp_in = fopen((char*)fname_in, "r");
  if (fp_in == (FILE*)NULL) {
    fprintf(stdout, "Unable to open file for reading\n");
    exit(ISIP_ERROR);
  }
  
  FILE* fp_out = fopen((char*)fname_out, "w");
  if (fp_out == (FILE*)NULL) {
    fprintf(stdout, "Unable to open file for writing\n");
    exit(ISIP_ERROR);
  }
  
  char_1* buff = new char_1[ISIP_MAX_STRING_LENGTH];
  char_1* letter = new char_1[R_MAX_WORD*max];
  char_1** phone = new char_1*[R_MAX_WORD*max];
  float_4* score = new float_4[R_MAX_WORD*max];
  int_4 finish = ISIP_FALSE;
  int_4 count = (int_4)0;
  char_1* nonwhitespace = new char_1[R_MAX_PHONE];
  
  for (int_4 i=0; i < R_MAX_WORD*max; i++) {
    phone[i] = new char_1[R_MAX_PHONE];
  }
  
  while (!finish) {

    if (fgets((char*)buff, ISIP_MAX_STRING_LENGTH, fp_in) != NULL) {
      buff[strlen((char*)buff)-1] = ISIP_NULL;
      
      // get next context
      //
      if (sscanf((char*)buff, "%1s", nonwhitespace) > 0) {
	if (sscanf((char*)buff, "%c%s%f", &letter[count], phone[count],
		   &score[count]) == 3) {
	  count++;
	}
	else {
	  fprintf(stdout, "bad format\n");
	  exit(ISIP_ERROR);
	}
      }
      
      // process this word before going to the next word
      //
      else {
	
	// reset count
	//
	int_4 len = count/max;
	count = (int_4)0;

	// construct and output the word spelling
	//
	strcpy((char*)buff, "* ");
	sprintf((char*)buff, "%s%c", buff, toupper(letter[0]));
	for (int i = 1; i < len; i++) {
	  if (letter[(i - 1) * max] != ' ') {
	    sprintf((char*)buff, "%s%c", buff, letter[i * max]);
	  }
	  else {
	    sprintf((char*)buff, "%s%c", buff, letter[i * max]);
	  }
	}
	fprintf(fp_out, "%s\n", buff);
	
	// there are max**len strings, but limit them to a reasonable
	// value
	//
	int_4 num_strings = (int_4)pow(max, len);
	if (num_strings > R_MAX_STRINGS) {
	  num_strings = R_MAX_STRINGS;
	}
	
	// allocate space for pronunciations and scores
	//
	int_4 str_count = 0;
	char_1** out_str = new char_1*[num_strings];
	float_4* out_score = new float_4[num_strings];
	
	// initialize counter indexing
	//
	int_4 *index = new int_4[len];
	for (int_4 i = 0; i < len; i++) {
	  index[i] = (int_4)0;
	}
	
	// construct each pronunciation and combine scores
	//
	char_1* tmp_str = new char_1[ISIP_MAX_STRING_LENGTH];
	float_4 tmp_score = (float_4)0;
	while (str_count < num_strings) {
	  
	  // construct a pronunciation string
	  //
	  tmp_score = (float_4)0;
	  memset(tmp_str, 0, sizeof(char_1)*ISIP_MAX_STRING_LENGTH);
	  for (int_4 l = 0; l < len; l++) {
	    int_4 jj = len - 1 - l;
	    strcat((char*)tmp_str, (char*)phone[l * max + index[jj]]);
	    tmp_score += log(score[l * max + index[jj]]);
	    strcat((char*)tmp_str, " ");
	    index[jj] = (int_4)0;
	  }
	  
	  // set the new pronunciation string and score
	  //
	  out_str[str_count] = new char_1[strlen((char*)tmp_str)+1];
	  strcpy((char*)out_str[str_count], (char*)tmp_str);
	  out_score[str_count] = tmp_score;
	  str_count++;
	  
	  // modify index counters
	  //
	  int_4 pquot = str_count;
	  int_4 quot = (int_4)0;
	  for (int_4 ii = 0; ii < len; ii++) {
	    quot = pquot / max;
	    index[ii] = pquot - quot * max;
	    pquot = quot;
	  }
	}
	
	// sort all the strings in decreasing order of scores
	//
	int_4* order = new int_4[num_strings];	
	qsort_cc(order, out_score, num_strings);
	
	// output only the top nbest strings and their scores to output file
	//
	logical_1 repeat = ISIP_FALSE;
	for (int_4 n = 0, num_out = 0;
	     (n < num_strings) && (num_out < nbest); n++) {
	  
	  // check if we have already output this pronunciation
	  //
	  repeat = ISIP_FALSE;
	  for (int_4 p = 0; p < n; p++) {
	    if (strcmp((char*)out_str[order[n]],
		       (char*)out_str[order[p]]) == 0) {
	      repeat = ISIP_TRUE;
	      break;
	    }
	  }
	  if (repeat == ISIP_FALSE) {
	    num_out ++;
	    fprintf(fp_out, "%s\tScore: %f\n", out_str[order[n]],
		    out_score[order[n]]);
	  }
	}
	fprintf(fp_out, "\n");
	
	// free memory
	//
	delete index;
	delete order;
	delete out_score;
        for (int_4 m = 0; m < str_count; m++) {
          delete [] out_str[m];
        }
	delete [] out_str;
	delete [] tmp_str;
      }
    }
    else {
      finish = 1;
    }
  }
  
  // free memory
  //
  delete [] fname_in;
  delete [] fname_out;
  for (int_4 m = 0; m < R_MAX_WORD * max; m++) {
    delete [] phone[m];
  }
  delete [] phone;
  delete [] buff;
  delete [] letter;
  delete [] nonwhitespace;
  delete score;
  
  // close files
  //
  fclose(fp_in);
  fclose(fp_out);

  // exit gracefully
  //
  exit(ISIP_NO_ERROR);
}
